/*
** Copyright 1995-1996 by the Massachusetts Institute of Technology.  All
** rights reserved. For more information, see the file COPYING which came
** with this source distribution.
*/

// This parser is meant to be used with the default output looped back
// to the default input. It sends out MIDI messages and measures the amount
// of time it takes for them to get back

#include "PingParser.h"
#include "data_structs.h"
#include "ParserRegistration.h"
#include "sched.h"

static Bool running;
static ioref ou_node;

static int count;
static ScheduleTime lastsent;

static void infunc(MIDI_msg*)
{
	if (lastsent == NEVER)
		tw.pr("extraneous message!\n");
	ScheduleTime t = oap->time();
	tw.pr("message: %d time: %d demimilliseconds\n", count++, t-lastsent);
	lastsent = NEVER;
}

static ScheduleTime func(void*)
{
	if (!running)
		return NEVER;
	if (lastsent != NEVER)
		tw.pr("Dropped message %d\n",count++);
	ScheduleTime t = oap->time();
	MIDI_msg m;
	m.set(((count/2)%2)?MIDI_msg::note_off:MIDI_msg::note_on, 0, 60+(count%2), 64, ou_node);
	oap->write(&m);
	lastsent = t;
	return 2000;
}

PingParser::
PingParser()
{
	msg_hnd[MIDI_msg::note_on] = infunc;
	msg_hnd[MIDI_msg::note_off] = infunc;
	ou_node = oap->open_output(); // open the default output
	oap->open_input(); // open the default input
	running = True;
	count = 0;
	lastsent = NEVER;
	task_data td(func, NULL, oap->time()+2000);
	schedp->insert(td);
}

PingParser::
~PingParser()
{
	running = False;
}

static parser* newFunc()
{
	return (new PingParser());
}

static ParserRegistration reg(newFunc, "PingParser");

// Author: Patrick Pelletier
