/*
** Copyright 1995-1996 by the Massachusetts Institute of Technology.  All
** rights reserved. For more information, see the file COPYING which came
** with this source distribution.
*/

// This parser is meant to be used with the default output looped back
// to the default input. It sends out sysex MIDI messages and measures the amount
// of time it takes for them to get back

#include "OMS_app.h"
#include "MIDI_lib.h"
#include "data_structs.h"
#include "ParserRegistration.h"
#include "sched.h"

class SysexPingParser : public parser
{
public:
	SysexPingParser();
	~SysexPingParser();
};

static Bool running;
static ioref ou_node;

static int count;
static ScheduleTime lastsent;

static void infunc(MIDI_msg*)
{
	if (lastsent == NEVER)
		tw.pr("extraneous message!\n");
	ScheduleTime t = oap->time();
	tw.pr("message: %d time: %d demimilliseconds\n", count++, t-lastsent);
	lastsent = NEVER;
}

static ScheduleTime func(void*)
{
//	const unsigned char sample_sysex_data[] = { 0xf0, 0x7d, 'P', 'a', 't', 'r', 'i', 'c', 'k', 0xf7 };
	const unsigned char sample_sysex_data[] = { 0xf0, 0x7d, 0xf7 };
	if (!running)
		return NEVER;
	if (lastsent != NEVER)
		tw.pr("Dropped message %d\n",count++);
	ScheduleTime t = oap->time();
	MIDI_msg m;
	m.node = ou_node;
//	m.set_sys_ex(sample_sysex_data, 10);
	m.set_sys_ex(sample_sysex_data, 3);
	oap->write(&m);
	lastsent = t;
	return 2000;
}

SysexPingParser::
SysexPingParser()
{
	msg_hnd[MIDI_msg::system] = infunc;
	ou_node = oap->open_output(); // open the default output
	oap->open_input(); // open the default input
	running = True;
	count = 0;
	lastsent = NEVER;
	task_data td(func, NULL, oap->time()+2000);
	schedp->insert(td);
}

SysexPingParser::
~SysexPingParser()
{
	running = False;
}

static parser* newFunc()
{
	return (new SysexPingParser());
}

static ParserRegistration reg(newFunc, "SysexPingParser");

// Author: Patrick Pelletier
